#pragma once

#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <string>
#include <vector>
#include <errno.h>
#include <sys/stat.h>
#ifdef _WIN32
#include <direct.h>
#endif

#include "minizip/unzip.h"
#include "GakumasLocalify/Log.h"

// ݹ鴴Ŀ¼
static bool CreateDirectoryRecursively(const std::string& dir) {
    if (dir.empty())
        return false;

    // ԴĿ¼
#ifdef _WIN32
    int ret = _mkdir(dir.c_str());
#else
    int ret = mkdir(dir.c_str(), 0755);
#endif
    if (ret == 0 || errno == EEXIST)
        return true;

    // ʧܣȴĿ¼
    size_t pos = dir.find_last_of("/\\");
    if (pos != std::string::npos) {
        std::string parentDir = dir.substr(0, pos);
        if (!CreateDirectoryRecursively(parentDir))
            return false;
#ifdef _WIN32
        ret = _mkdir(dir.c_str());
#else
        ret = mkdir(dir.c_str(), 0755);
#endif
        return (ret == 0 || errno == EEXIST);
    }
    return false;
}

// ѹ
static bool UnzipFile(const std::string& zipPath, const std::string& destinationFolder) {
    // zipļ
    unzFile zipfile = unzOpen(zipPath.c_str());
    if (zipfile == nullptr) {
        GakumasLocal::Log::ErrorFmt( "Can't open zip file: %s", zipPath.c_str());
        return false;
    }

    int ret = unzGoToFirstFile(zipfile);
    if (ret != UNZ_OK) {
        GakumasLocal::Log::ErrorFmt("Can't read first file %s", zipPath.c_str());
        unzClose(zipfile);
        return false;
    }

    // zipڵÿļ
    do {
        char filename[512] = { 0 };
        unz_file_info fileInfo;
        ret = unzGetCurrentFileInfo(zipfile, &fileInfo,
            filename, sizeof(filename),
            nullptr, 0, nullptr, 0);
        if (ret != UNZ_OK) {
            GakumasLocal::Log::ErrorFmt("Read ZIP File Info Error");
            unzClose(zipfile);
            return false;
        }

        std::string filePath = filename;
        std::string fullPath = destinationFolder;
        // ֤ĿĿ¼·ָβ
        if (fullPath.back() != '/' && fullPath.back() != '\\') {
            fullPath += "/";
        }
        fullPath += filePath;

        // жǷΪĿ¼Ŀ¼Ŀͨ'/'β
        if (!filePath.empty() && (filePath.back() == '/' || filePath.back() == '\\')) {
            // Ŀ¼
            if (!CreateDirectoryRecursively(fullPath)) {
                GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", fullPath.c_str());
                unzClose(zipfile);
                return false;
            }
        }
        else {
            // ļȷϼĿ¼
            size_t pos = fullPath.find_last_of("/\\");
            if (pos != std::string::npos) {
                std::string directory = fullPath.substr(0, pos);
                if (!CreateDirectoryRecursively(directory)) {
                    GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", directory.c_str());
                    unzClose(zipfile);
                    return false;
                }
            }

            // zipļ
            ret = unzOpenCurrentFile(zipfile);
            if (ret != UNZ_OK) {
                GakumasLocal::Log::ErrorFmt("Open file in zip failed: %s", filePath.c_str());
                unzClose(zipfile);
                return false;
            }

            // Ŀ·ϴļ
            FILE* outFile = fopen(fullPath.c_str(), "wb");
            if (outFile == nullptr) {
                GakumasLocal::Log::ErrorFmt("Can't create output file: %s", fullPath.c_str());
                unzCloseCurrentFile(zipfile);
                unzClose(zipfile);
                return false;
            }

            // ȡݲдļ
            const int bufferSize = 8192;
            std::vector<char> buffer(bufferSize);
            int bytesRead = 0;
            do {
                bytesRead = unzReadCurrentFile(zipfile, buffer.data(), bufferSize);
                if (bytesRead < 0) {
                    GakumasLocal::Log::ErrorFmt("Read File Error: %s", filePath.c_str());
                    fclose(outFile);
                    unzCloseCurrentFile(zipfile);
                    unzClose(zipfile);
                    return false;
                }
                if (bytesRead > 0) {
                    fwrite(buffer.data(), 1, bytesRead, outFile);
                }
            } while (bytesRead > 0);

            fclose(outFile);
            unzCloseCurrentFile(zipfile);
        }

        ret = unzGoToNextFile(zipfile);
    } while (ret == UNZ_OK);

    unzClose(zipfile);
    return true;
}


static bool UnzipFile(const std::string& zipPath, const std::string& destinationFolder, std::string_view targetDir) {
	if (targetDir.empty()) {
		return UnzipFile(zipPath, destinationFolder);
	}

    //  zip ļ
    unzFile zipfile = unzOpen(zipPath.c_str());
    if (zipfile == nullptr) {
        GakumasLocal::Log::ErrorFmt("Can't open zip file: %s", zipPath.c_str());
        return false;
    }

    // ڱʶZIP Ŀ¼ǷֱӰ targetDir ļ
    bool targetAtRoot = false;
    // ڸĿ¼ûҵٳԼ¼ĸĿ¼° targetDirֻһ㣩
    std::string candidateRoot;

    // һɨ裺ĿжĿļλ
    int ret = unzGoToFirstFile(zipfile);
    if (ret != UNZ_OK) {
        GakumasLocal::Log::ErrorFmt("Can't read first file: %s", zipPath.c_str());
        unzClose(zipfile);
        return false;
    }
    do {
        char filename[512] = { 0 };
        unz_file_info fileInfo{};
        ret = unzGetCurrentFileInfo(zipfile, &fileInfo,
            filename, sizeof(filename),
            nullptr, 0, nullptr, 0);
        if (ret != UNZ_OK) {
            GakumasLocal::Log::ErrorFmt("Read ZIP File Info Error");
            unzClose(zipfile);
            return false;
        }
        std::string entryName = filename;
        //  '\' ͳһ滻Ϊ '/'
        std::replace(entryName.begin(), entryName.end(), '\\', '/');

        // Ŀȫ targetDirԴ򲻴β '/'ûǰ׺ΪڸĿ¼
        if ((entryName == targetDir || entryName == std::string(targetDir) + "/") &&
            entryName.find('/') == entryName.size() - 1) {
            targetAtRoot = true;
        }
        else {
            // ں·ָĿȡһĿ¼
            if (auto pos = entryName.find('/'); pos != std::string::npos) {
                std::string root = entryName.substr(0, pos);
                // remainder Ϊ root ·
                std::string remainder = entryName.substr(pos + 1);
                // һĿ¼
                auto pos2 = remainder.find('/');
                std::string firstSubDir = (pos2 != std::string::npos) ? remainder.substr(0, pos2) : remainder;
                if (firstSubDir == targetDir) {
                    candidateRoot = root;
                }
            }
        }
        ret = unzGoToNextFile(zipfile);
    } while (ret == UNZ_OK);

    // ɨȷ extractionPrefix
    // ĿļڸĿ¼ѹ targetDir ڵĿдʱȥ targetDir ǰ׺
    //  candidateRoot Ϊգѹ candidateRoot ڵĿдʱȥ candidateRoot ǰ׺
    std::string extractionPrefix;
    if (targetAtRoot) {
        extractionPrefix = std::string(targetDir) + "/";
    }
    else if (!candidateRoot.empty()) {
        extractionPrefix = candidateRoot + "/";
    }
    else {
        GakumasLocal::Log::ErrorFmt("Target directory '%.*s' not found in zip", (int)targetDir.size(), targetDir.data());
        unzClose(zipfile);
        return false;
    }

    // ´ zip ļнѹɨλã
    unzClose(zipfile);
    zipfile = unzOpen(zipPath.c_str());
    if (zipfile == nullptr) {
        GakumasLocal::Log::ErrorFmt("Can't reopen zip file: %s", zipPath.c_str());
        return false;
    }
    ret = unzGoToFirstFile(zipfile);
    if (ret != UNZ_OK) {
        GakumasLocal::Log::ErrorFmt("Can't read first file: %s", zipPath.c_str());
        unzClose(zipfile);
        return false;
    }

    // ڶɨ裺ѹָĿ¼ڵĿ
    do {
        char filename[512] = { 0 };
        unz_file_info fileInfo{};
        ret = unzGetCurrentFileInfo(zipfile, &fileInfo,
            filename, sizeof(filename),
            nullptr, 0, nullptr, 0);
        if (ret != UNZ_OK) {
            GakumasLocal::Log::ErrorFmt("Read ZIP File Info Error");
            unzClose(zipfile);
            return false;
        }
        std::string entryName = filename;
        std::replace(entryName.begin(), entryName.end(), '\\', '/');

        //  extractionPrefix ͷĿ
        if (entryName.rfind(extractionPrefix, 0) == 0) {
            // ȥ extractionPrefix õ·
            std::string relativePath = entryName.substr(extractionPrefix.size());

            // ȫ·
            std::string fullPath = destinationFolder;
            if (!fullPath.empty() && fullPath.back() != '/' && fullPath.back() != '\\')
                fullPath += "/";
            fullPath += relativePath;

            //  relativePath Ϊջ '/' βΪĿ¼
            if (relativePath.empty() || relativePath.back() == '/') {
                if (!CreateDirectoryRecursively(fullPath)) {
                    GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", fullPath.c_str());
                    unzClose(zipfile);
                    return false;
                }
            }
            else {
                // ȷϼĿ¼
                if (auto pos = fullPath.find_last_of("/\\"); pos != std::string::npos) {
                    std::string directory = fullPath.substr(0, pos);
                    if (!CreateDirectoryRecursively(directory)) {
                        GakumasLocal::Log::ErrorFmt("Create Dir Failed: %s", directory.c_str());
                        unzClose(zipfile);
                        return false;
                    }
                }
                ret = unzOpenCurrentFile(zipfile);
                if (ret != UNZ_OK) {
                    GakumasLocal::Log::ErrorFmt("Open file in zip failed: %s", entryName.c_str());
                    unzClose(zipfile);
                    return false;
                }
                FILE* outFile = fopen(fullPath.c_str(), "wb");
                if (outFile == nullptr) {
                    GakumasLocal::Log::ErrorFmt("Can't create output file: %s", fullPath.c_str());
                    unzCloseCurrentFile(zipfile);
                    unzClose(zipfile);
                    return false;
                }
                constexpr int bufferSize = 8192;
                std::vector<char> buffer(bufferSize);
                int bytesRead = 0;
                while ((bytesRead = unzReadCurrentFile(zipfile, buffer.data(), bufferSize)) > 0) {
                    if (fwrite(buffer.data(), 1, bytesRead, outFile) != static_cast<size_t>(bytesRead)) {
                        GakumasLocal::Log::ErrorFmt("Write File Error: %s", fullPath.c_str());
                        fclose(outFile);
                        unzCloseCurrentFile(zipfile);
                        unzClose(zipfile);
                        return false;
                    }
                }
                if (bytesRead < 0) {
                    GakumasLocal::Log::ErrorFmt("Read File Error: %s", entryName.c_str());
                    fclose(outFile);
                    unzCloseCurrentFile(zipfile);
                    unzClose(zipfile);
                    return false;
                }
                fclose(outFile);
                unzCloseCurrentFile(zipfile);
            }
        }
        ret = unzGoToNextFile(zipfile);
    } while (ret == UNZ_OK);
    unzClose(zipfile);
    return true;
}
